@@ -10,6 +10,10 @@ case "`uname`" in
     FreeBSD)
         RESOLVED_NAME=`readlink -f "$0"`
         ;;
+    OpenBSD)
+        RESOLVED_NAME=`readlink -f "$0"`
+        JAVA_HOME=`/usr/local/bin/javaPathHelper -h keycloak`
+        ;;
     Linux)
         RESOLVED_NAME=`readlink -f "$0"`
         ;;
@@ -30,4 +34,4 @@ if [ "x$JAVA" = "x" ]; then
     fi
 fi
 
-"$JAVA" $KC_OPTS -cp $DIRNAME/client/keycloak-admin-cli-20.0.0.jar org.keycloak.client.admin.cli.KcAdmMain "$@"
+"$JAVA" $KC_OPTS -cp $DIRNAME/client/keycloak-admin-cli-22.0.1.jar --add-opens=java.base/java.security=ALL-UNNAMED -Dkc.lib.dir=$DIRNAME/client/lib org.keycloak.client.admin.cli.KcAdmMain "$@"
